Attribute VB_Name = "Module1"
'//
'// Module Screen control
'//

'// Function GetScreenMode() As Boolean
'// (Ret = False: Error; Ret = True: Ok)
'//
'//         Data saved in OriginalScreenMode as ScreenInfo type:
'//         Private Type ScreenInfo
'//             siWidth As Long
'//             siHeight As Long
'//             siBPP As Integer
'//         End Type
'//
'// Function SetScreenMode(Width As Long, Height As Long, BPP As Integer) As Boolean
'// (Ret = False: Error; Ret = True: Ok)
'//
'//         Only Ret = True if dynamic change is successful

Private Declare Function GetSystemMetrics Lib "user32" _
                        (ByVal nIndex As Long) As Long

Private Declare Function CreateIC Lib "gdi32" Alias "CreateICA" _
                        (ByVal lpDriverName As String, _
                         ByVal lpDeviceName As String, _
                         ByVal lpOutput As String, _
                         lpInitData As DEVMODE) As Long
        
Private Declare Function GetDeviceCaps Lib "gdi32" _
                        (ByVal hdc As Long, _
                         ByVal nIndex As Long) As Long
        
Private Declare Function DeleteDC Lib "gdi32" _
                        (ByVal hdc As Long) As Long
       
Private Declare Function EnumDisplaySettings Lib "user32" Alias "EnumDisplaySettingsA" _
                        (ByVal lpszDeviceName As Long, _
                         ByVal iModeNum As Long, _
                         lpDevMode As Any) As Boolean
        
Private Declare Function ChangeDisplaySettings Lib "user32" Alias "ChangeDisplaySettingsA" _
                        (lpDevMode As Any, _
                         ByVal dwFlags As Long) As Long
    
Private Const CCDEVICENAME = 32
Private Const CCFORMNAME = 32

Private Const SM_CXSCREEN = 0
Private Const SM_CYSCREEN = 1
Private Const BITSPIXEL = 12
  
Private Const CDS_UPDATEREGISTRY = &H1
Private Const CDS_TEST = &H4

Private Const DISP_CHANGE_SUCCESSFUL = 0

Private Type DEVMODE
    dmDeviceName As String * CCDEVICENAME
    dmSpecVersion As Integer
    dmDriverVersion As Integer
    dmSize As Integer
    dmDriverExtra As Integer
    dmFields As Long
    dmOrientation As Integer
    dmPaperSize As Integer
    dmPaperLength As Integer
    dmPaperWidth As Integer
    dmScale As Integer
    dmCopies As Integer
    dmDefaultSource As Integer
    dmPrintQuality As Integer
    dmColor As Integer
    dmDuplex As Integer
    dmYResolution As Integer
    dmTTOption As Integer
    dmCollate As Integer
    dmFormName As String * CCFORMNAME
    dmUnusedPadding As Integer
    dmBitsPerPel As Integer
    dmPelsWidth As Long
    dmPelsHeight As Long
    dmDisplayFlags As Long
    dmDisplayFrequency As Long
End Type

'//

' Screen resolution and color depth:
Private Type ScreenInfo
    siWidth As Long
    siHeight As Long
    siBPP As Integer
End Type

' It will be used to restore screen mode:
Public OriginalScreenMode As ScreenInfo

Function GetScreenMode() As Boolean
    
    Dim lHDC As Long, tDevMode As DEVMODE
    
    ' Get Screen resolution
    OriginalScreenMode.siWidth = GetSystemMetrics(SM_CXSCREEN)
    OriginalScreenMode.siHeight = GetSystemMetrics(SM_CYSCREEN)
    
    ' Get Colour Depth (4, 8, 16, 24, 32 bpp)
    lHDC = CreateIC("DISPLAY", 0&, 0&, tDevMode)     ' Create an information context device
                                                     ' without create the context device
    If lHDC = 0 Then
        GetScreenMode = False                        ' An error has ocurred
        Exit Function
    End If
    
    OriginalScreenMode.siBPP = GetDeviceCaps(lHDC, BITSPIXEL)
    
    lHDC = DeleteDC(lHDC)
    
    GetScreenMode = True
    
End Function

Function SetScreenMode(Width As Long, Height As Long, BPP As Integer) As Boolean

    Dim tDevMode As DEVMODE, lRet As Long, lModeIndex As Long
    
    lModeIndex = 0
    
    Do
        lRet = EnumDisplaySettings(0&, lModeIndex, tDevMode)
        
        If lRet = 0 Then            ' All called modes or an error has ocurred
            SetScreenMode = False
            Exit Function
        End If
                                    
        With tDevMode
            If .dmPelsWidth = Width And _
               .dmPelsHeight = Height And _
               .dmBitsPerPel = BPP Then
                ' The called mode exists:
                ' Try to change mode dynamicaly if possible (CDS_UPDATEREGISTRY)
                ' If not possible, update registry for change on the next boot-up
                ' CDS_TEST constant can be used to test before set new settings
                lRet = ChangeDisplaySettings(tDevMode, CDS_UPDATEREGISTRY)
                Exit Do
            End If
        End With
        
        lModeIndex = lModeIndex + 1 ' Increase index (next mode available)
        
    Loop
    
    If lRet = DISP_CHANGE_SUCCESSFUL Then
        SetScreenMode = True        ' Mode successfuly changed dynamicaly
    Else
        SetScreenMode = False       ' Unable to change resolution or
    End If                          ' a reboot is necessary before the changes will take effect
                
End Function

